"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createConfigFile;

var _get2 = _interopRequireDefault(require("lodash/get"));

var _mapValues2 = _interopRequireDefault(require("lodash/mapValues"));

var _template2 = _interopRequireDefault(require("lodash/template"));

var _reduce2 = _interopRequireDefault(require("lodash/reduce"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _chalk = _interopRequireDefault(require("chalk"));

var _files = require("../utils/files");

var _logger = require("../utils/logger");

var _ci = require("../utils/ci");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const {
  CI_ENVIRONMENT_SLUG
} = process.env;

function formattedErrorMessage(err) {
  const errMessage = (0, _get2.default)(err, 'message', 'Issue templating config file');

  if (!errMessage.includes('is not defined')) {
    return errMessage;
  }

  const splitMessage = err.message.split(' is not defined');
  return `${_chalk.default.cyan(splitMessage[0])} is not defined in environment`;
}

function tryTemplating(str, name) {
  const {
    version
  } = (0, _files.getFile)('package.json');

  try {
    return (0, _template2.default)(str)(_objectSpread({}, process.env, {
      version,
      npm_package_version: version
    }));
  } catch (err) {
    const errMsg = formattedErrorMessage(err);
    (0, _logger.warn)(`${errMsg}. Setting ${_chalk.default.cyan(name)} to an empty string.`);
    return '';
  }
}

function createConfigFile(config) {
  const settings = (0, _files.getFile)('.firebaserc');

  if (!settings) {
    (0, _logger.error)('.firebaserc file is required');
    throw new Error('.firebaserc file is required');
  }

  if (!settings.ci || !settings.ci.createConfig) {
    (0, _logger.error)('no createConfig settings found');
    return;
  }

  const opts = {
    path: (0, _get2.default)(config, 'path', './src/config.js'),
    project: (0, _ci.getProjectKey)(config)
  };
  const {
    ci: {
      createConfig
    }
  } = settings;
  const fallBackConfigName = CI_ENVIRONMENT_SLUG || (createConfig.master ? 'master' : 'default');
  const envConfig = createConfig[opts.project] || createConfig[fallBackConfigName];

  if (!envConfig) {
    const msg = 'Valid create config settings could not be loaded';
    (0, _logger.error)(msg);
    throw new Error(msg);
  }

  (0, _logger.info)(`Creating config file at ${_chalk.default.cyan(opts.path)} for project: ${_chalk.default.cyan(createConfig[opts.project] ? opts.project : fallBackConfigName)}`);
  const templatedData = (0, _mapValues2.default)(envConfig, (parent, parentName) => typeof parent === 'string' ? tryTemplating(parent, parentName) : (0, _mapValues2.default)(parent, (data, childKey) => tryTemplating(data, `${parentName}.${childKey}`)));

  const parentAsString = parent => (0, _reduce2.default)(parent, (acc, child, childKey) => acc.concat(`  ${childKey}: ${JSON.stringify(child, null, 2)},\n`), '');

  const exportString = _path.default.extname(opts.path) === '.json' ? JSON.stringify(templatedData, null, 2) : (0, _reduce2.default)(templatedData, (acc, parent, parentName) => acc.concat(`export const ${parentName} = `).concat(typeof parent === 'string' ? `"${parent}";\n\n` : `{\n${parentAsString(parent)}};\n\n`), '').concat(`export default { ${Object.keys(templatedData).join(', ')} };\n`);

  const folderName = _path.default.basename(_path.default.dirname(opts.path));

  if (!_fs.default.existsSync(`./${folderName}`)) {
    _fs.default.mkdirSync(folderName);
  }

  try {
    _fs.default.writeFileSync(opts.path, exportString, 'utf8');
  } catch (err) {
    (0, _logger.error)('Error creating config file: ', err);
  }
}