"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = setEnvConfig;

var _mapValues2 = _interopRequireDefault(require("lodash/mapValues"));

var _template2 = _interopRequireDefault(require("lodash/template"));

var _path = _interopRequireDefault(require("path"));

var _chalk = _interopRequireDefault(require("chalk"));

var _files = require("../utils/files");

var _logger = require("../utils/logger");

var _ci = require("../utils/ci");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const {
  CI_ENVIRONMENT_SLUG
} = process.env;

function tryTemplating(str, name) {
  const {
    version
  } = (0, _files.getFile)('package.json');

  try {
    return (0, _template2.default)(str)(_objectSpread({}, process.env, {
      version,
      npm_package_version: version
    }));
  } catch (err) {
    (0, _logger.warn)(`${err.message}. Setting ${_chalk.default.cyan(name)} to an empty string.`);
    return '';
  }
}

async function setEnvConfig(config) {
  var _settings$ci;

  const settings = (0, _files.getFile)('.firebaserc');

  if (!settings) {
    (0, _logger.error)('.firebaserc file is required');
    throw new Error('.firebaserc file is required');
  }

  if (!((_settings$ci = settings.ci) === null || _settings$ci === void 0 ? void 0 : _settings$ci.setEnv)) {
    (0, _logger.error)('no setEnv settings found');
    return;
  }

  const selectedProject = (0, _ci.getProjectKey)(config);
  const {
    ci: {
      setEnv
    }
  } = settings;
  const fallBackConfigName = CI_ENVIRONMENT_SLUG || (setEnv.master ? 'master' : 'default');
  const envConfig = setEnv[selectedProject] || setEnv[fallBackConfigName];

  if (!envConfig) {
    const msg = 'Valid create config settings could not be loaded';
    (0, _logger.error)(msg);
    throw new Error(msg);
  }

  (0, _logger.info)(`Setting environment from config for project ${selectedProject}`);
  const templatedData = (0, _mapValues2.default)(envConfig, (parent, parentName) => typeof parent === 'string' ? tryTemplating(parent, parentName) : (0, _mapValues2.default)(parent, (data, childKey) => tryTemplating(data, `${parentName}.${childKey}`)));

  if (process.env.GITHUB_ACTIONS) {
    (0, _logger.info)(`Github actions environment detected setting-env will also be called`);
  }

  Object.keys(templatedData).forEach(currentKey => {
    process.env[currentKey] = templatedData[currentKey];

    if (process.env.GITHUB_ACTIONS) {
      console.log(`::set-env name=${currentKey}::${templatedData[currentKey]}`);
    }
  });
}